@@ -34,6 +34,7 @@
 
 #include "gstghostpad.h"
 #include "gstutils.h"
+#include "gsterror.h"
 #include "gstinfo.h"
 #include "gstparse.h"
 #include "gst-i18n-lib.h"
@@ -56,9 +57,9 @@ gst_util_dump_mem (const guchar * mem, guint size)
   i = j = 0;
   while (i < size) {
     if (g_ascii_isprint (mem[i]))
-      g_string_append_printf (chars, "%c", mem[i]);
+      g_string_append_c (chars, mem[i]);
     else
-      g_string_append_printf (chars, ".");
+      g_string_append_c (chars, '.');
 
     g_string_append_printf (string, "%02x ", mem[i]);
 
@@ -885,7 +886,7 @@ gst_element_request_compatible_pad (GstElement * element,
  * cases.
  *
  * Returns: the #GstPad to which a link can be made, or %NULL if one cannot be
- * found.
+ * found. gst_object_unref() after usage.
  */
 GstPad *
 gst_element_get_compatible_pad (GstElement * element, GstPad * pad,
@@ -1030,6 +1031,8 @@ gst_element_state_get_name (GstState state)
  * Gets a string representing the given state change result.
  *
  * Returns: a string with the name of the state change result.
+ *
+ * Since: 0.10.11
  */
 G_CONST_RETURN gchar *
 gst_element_state_change_return_get_name (GstStateChangeReturn state_ret)
@@ -1548,7 +1551,8 @@ gst_element_link_pads (GstElement * src, const gchar * srcpadname,
                   gst_element_get_request_pad (src, srctempl->name_template);
               destpad =
                   gst_element_get_request_pad (dest, desttempl->name_template);
-              if (pad_link_maybe_ghosting (srcpad, destpad)) {
+              if (srcpad && destpad
+                  && pad_link_maybe_ghosting (srcpad, destpad)) {
                 GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS,
                     "linked pad %s:%s to pad %s:%s",
                     GST_DEBUG_PAD_NAME (srcpad), GST_DEBUG_PAD_NAME (destpad));
@@ -1557,8 +1561,10 @@ gst_element_link_pads (GstElement * src, const gchar * srcpadname,
                 return TRUE;
               }
               /* it failed, so we release the request pads */
-              gst_element_release_request_pad (src, srcpad);
-              gst_element_release_request_pad (dest, destpad);
+              if (srcpad)
+                gst_element_release_request_pad (src, srcpad);
+              if (destpad)
+                gst_element_release_request_pad (dest, destpad);
             }
           }
         }
@@ -1651,6 +1657,7 @@ gst_element_link_pads_filtered (GstElement * src, const gchar * srcpadname,
  * Links @src to @dest. The link must be from source to
  * destination; the other direction will not be tried. The function looks for
  * existing pads that aren't linked yet. It will request new pads if necessary.
+ * Such pads need to be released manualy when unlinking.
  * If multiple links are possible, only one is established.
  *
  * Make sure you have added your elements to a bin or pipeline with
@@ -1800,6 +1807,9 @@ gst_element_unlink_many (GstElement * element_1, GstElement * element_2, ...)
  *
  * Unlinks all source pads of the source element with all sink pads
  * of the sink element to which they are linked.
+ *
+ * If the link has been made using gst_element_link(), it could have created an
+ * requestpad, which has to be released using gst_element_release_request_pad().
  */
 void
 gst_element_unlink (GstElement * src, GstElement * dest)
@@ -1965,7 +1975,8 @@ gst_element_query_convert (GstElement * element, GstFormat src_format,
  * gst_element_seek_simple
  * @element: a #GstElement to seek on
  * @format: a #GstFormat to execute the seek in, such as #GST_FORMAT_TIME
- * @seek_flags: seek options
+ * @seek_flags: seek options; playback applications will usually want to use
+ *            GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT here
  * @seek_pos: position to seek to (relative to the start); if you are doing
  *            a seek in #GST_FORMAT_TIME this value is in nanoseconds -
  *            multiply with #GST_SECOND to convert seconds to nanoseconds or
@@ -2184,7 +2195,8 @@ gst_object_default_error (GstObject * source, GError * error, gchar * debug)
  * @...: additional elements to add to the bin
  *
  * Adds a NULL-terminated list of elements to a bin.  This function is
- * equivalent to calling gst_bin_add() for each member of the list.
+ * equivalent to calling gst_bin_add() for each member of the list. The return
+ * value of each gst_bin_add() is ignored.
  */
 void
 gst_bin_add_many (GstBin * bin, GstElement * element_1, ...)
@@ -2413,19 +2425,19 @@ gst_buffer_join (GstBuffer * buf1, GstBuffer * buf2)
  * Copies additional information (the timestamp, duration, and offset start
  * and end) from one buffer to the other.
  *
- * This function does not copy any buffer flags or caps.
+ * This function does not copy any buffer flags or caps and is equivalent to
+ * gst_buffer_copy_metadata(@dest, @src, GST_BUFFER_COPY_TIMESTAMPS).
+ *
+ * Deprecated: use gst_buffer_copy_metadata() instead, it provides more
+ * control.
  */
+#ifndef GST_REMOVE_DEPRECATED
 void
 gst_buffer_stamp (GstBuffer * dest, const GstBuffer * src)
 {
-  g_return_if_fail (dest != NULL);
-  g_return_if_fail (src != NULL);
-
-  GST_BUFFER_TIMESTAMP (dest) = GST_BUFFER_TIMESTAMP (src);
-  GST_BUFFER_DURATION (dest) = GST_BUFFER_DURATION (src);
-  GST_BUFFER_OFFSET (dest) = GST_BUFFER_OFFSET (src);
-  GST_BUFFER_OFFSET_END (dest) = GST_BUFFER_OFFSET_END (src);
+  gst_buffer_copy_metadata (dest, src, GST_BUFFER_COPY_TIMESTAMPS);
 }
+#endif /* GST_REMOVE_DEPRECATED */
 
 static gboolean
 intersect_caps_func (GstPad * pad, GValue * ret, GstPad * orig)
@@ -2836,7 +2848,7 @@ gst_atomic_int_set (gint * atomic_int, gint value)
  * Adds a "data probe" to a pad. This function will be called whenever data
  * passes through a pad. In this case data means both events and buffers. The
  * probe will be called with the data as an argument, meaning @handler should
- * have the same callback signature as the 'have-data' signal of #GstPad.
+ * have the same callback signature as the #GstPad::have-data signal.
  * Note that the data will have a reference count greater than 1, so it will
  * be immutable -- you must not change it.
  *
@@ -2880,7 +2892,7 @@ gst_pad_add_data_probe (GstPad * pad, GCallback handler, gpointer data)
 /**
  * gst_pad_add_event_probe:
  * @pad: pad to add the event probe handler to
- * @handler: function to call when data is passed over pad
+ * @handler: function to call when events are passed over pad
  * @data: data to pass along with the handler
  *
  * Adds a probe that will be called for all events passing through a pad. See
@@ -2909,7 +2921,7 @@ gst_pad_add_event_probe (GstPad * pad, GCallback handler, gpointer data)
 /**
  * gst_pad_add_buffer_probe:
  * @pad: pad to add the buffer probe handler to
- * @handler: function to call when data is passed over pad
+ * @handler: function to call when buffers are passed over pad
  * @data: data to pass along with the handler
  *
  * Adds a probe that will be called for all buffers passing through a pad. See
@@ -3081,7 +3093,7 @@ element_find_unconnected_pad (GstElement * element, GstPadDirection direction)
       iter = gst_element_iterate_sink_pads (element);
       break;
     default:
-      g_assert_not_reached ();
+      g_return_val_if_reached (NULL);
   }
 
   done = FALSE;
@@ -3179,7 +3191,6 @@ gst_bin_find_unconnected_pad (GstBin * bin, GstPadDirection direction)
   return pad;
 }
 
-#ifndef GST_DISABLE_PARSE
 /**
  * gst_parse_bin_from_description:
  * @bin_description: command line describing the bin
@@ -3206,6 +3217,7 @@ GstElement *
 gst_parse_bin_from_description (const gchar * bin_description,
     gboolean ghost_unconnected_pads, GError ** err)
 {
+#ifndef GST_DISABLE_PARSE
   GstPad *pad = NULL;
   GstBin *bin;
   gchar *desc;
@@ -3239,5 +3251,111 @@ gst_parse_bin_from_description (const gchar * bin_description,
   }
 
   return GST_ELEMENT (bin);
-}
+#else
+  gchar *msg;
+
+  GST_WARNING ("Disabled API called: gst_parse_bin_from_description()");
+
+  msg = gst_error_get_message (GST_CORE_ERROR, GST_CORE_ERROR_DISABLED);
+  g_set_error (err, GST_CORE_ERROR, GST_CORE_ERROR_DISABLED, "%s", msg);
+  g_free (msg);
+
+  return NULL;
 #endif
+}
+
+/**
+ * gst_type_register_static_full:
+ * @parent_type: The GType of the parent type the newly registered type will 
+ *   derive from
+ * @type_name: NULL-terminated string used as the name of the new type
+ * @class_size: Size of the class structure.
+ * @base_init: Location of the base initialization function (optional).
+ * @base_finalize: Location of the base finalization function (optional).
+ * @class_init: Location of the class initialization function for class types 
+ *   Location of the default vtable inititalization function for interface 
+ *   types. (optional)
+ * @class_finalize: Location of the class finalization function for class types.
+ *   Location of the default vtable finalization function for interface types. 
+ *   (optional)
+ * @class_data: User-supplied data passed to the class init/finalize functions.
+ * @instance_size: Size of the instance (object) structure (required for 
+ *   instantiatable types only).
+ * @n_preallocs: The number of pre-allocated (cached) instances to reserve 
+ *   memory for (0 indicates no caching). Ignored on recent GLib's.
+ * @instance_init: Location of the instance initialization function (optional, 
+ *   for instantiatable types only).
+ * @value_table: A GTypeValueTable function table for generic handling of 
+ *   GValues of this type (usually only useful for fundamental types). 
+ * @flags: #GTypeFlags for this GType. E.g: G_TYPE_FLAG_ABSTRACT
+ *
+ * Helper function which constructs a #GTypeInfo structure and registers a 
+ * GType, but which generates less linker overhead than a static const 
+ * #GTypeInfo structure. For further details of the parameters, please see
+ * #GTypeInfo in the GLib documentation.
+ *
+ * Registers type_name as the name of a new static type derived from 
+ * parent_type. The value of flags determines the nature (e.g. abstract or 
+ * not) of the type. It works by filling a GTypeInfo struct and calling 
+ * g_type_info_register_static().
+ *
+ * Returns: A #GType for the newly-registered type.
+ *
+ * Since: 0.10.14
+ */
+GType
+gst_type_register_static_full (GType parent_type,
+    const gchar * type_name,
+    guint class_size,
+    GBaseInitFunc base_init,
+    GBaseFinalizeFunc base_finalize,
+    GClassInitFunc class_init,
+    GClassFinalizeFunc class_finalize,
+    gconstpointer class_data,
+    guint instance_size,
+    guint16 n_preallocs,
+    GInstanceInitFunc instance_init,
+    const GTypeValueTable * value_table, GTypeFlags flags)
+{
+  GTypeInfo info;
+
+  info.class_size = class_size;
+  info.base_init = base_init;
+  info.base_finalize = base_finalize;
+  info.class_init = class_init;
+  info.class_finalize = class_finalize;
+  info.class_data = class_data;
+  info.instance_size = instance_size;
+  info.n_preallocs = n_preallocs;
+  info.instance_init = instance_init;
+  info.value_table = value_table;
+
+  return g_type_register_static (parent_type, type_name, &info, flags);
+}
+
+
+/**
+ * gst_util_get_timestamp:
+ *
+ * Get a timestamp as GstClockTime to be used for interval meassurements.
+ * The timestamp should not be interpreted in any other way.
+ *
+ * Returns: the timestamp
+ *
+ * Since: 0.10.16
+ */
+GstClockTime
+gst_util_get_timestamp (void)
+{
+#if defined (HAVE_POSIX_TIMERS) && defined(HAVE_MONOTONIC_CLOCK)
+  struct timespec now;
+
+  clock_gettime (CLOCK_MONOTONIC, &now);
+  return GST_TIMESPEC_TO_TIME (now);
+#else
+  GTimeVal now;
+
+  g_get_current_time (&now);
+  return GST_TIMEVAL_TO_TIME (now);
+#endif
+}
