@@ -35,7 +35,7 @@
   (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_FILE_INDEX, GstFileIndexClass))
 #define GST_IS_FILE_INDEX(obj)          \
   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_FILE_INDEX))
-#define GST_IS_FILE_INDEX_CLASS(obj)    \
+#define GST_IS_FILE_INDEX_CLASS(klass)    \
   (GST_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_FILE_INDEX))
 
 /*
@@ -144,7 +144,7 @@ static GstIndexEntry *gst_file_index_get_assoc_entry (GstIndex * index, gint id,
 
 static GstIndex *parent_class = NULL;
 
-GType
+static GType
 gst_file_index_get_type (void)
 {
   static GType file_index_type = 0;
@@ -261,7 +261,7 @@ struct fi_find_writer_context
   GstFileIndexId *ii;
 };
 
-void
+static void
 _fi_find_writer (gpointer key, gpointer val, gpointer data)
 {
   struct fi_find_writer_context *cx = data;
@@ -395,7 +395,7 @@ gst_file_index_load (GstFileIndex * index)
         g_free (path);
         if (fd < 0) {
           GST_ERROR_OBJECT (index,
-              "Can't open '%s': %s", path, strerror (errno));
+              "Can't open '%s': %s", path, g_strerror (errno));
           continue;
         }
 
@@ -441,7 +441,7 @@ gst_file_index_load (GstFileIndex * index)
         close (fd);
         if (array_data == MAP_FAILED) {
           GST_ERROR_OBJECT (index,
-              "mmap %s failed: %s", path, strerror (errno));
+              "mmap %s failed: %s", path, g_strerror (errno));
           continue;
         }
 
@@ -495,7 +495,7 @@ static void
 _file_index_id_save_xml (gpointer _key, GstFileIndexId * ii, xmlNodePtr writers)
 {
   const gint bufsize = 16;
-  gchar buf[bufsize];
+  gchar buf[16];
   xmlNodePtr writer;
   xmlNodePtr formats;
   gint xx;
@@ -601,7 +601,8 @@ gst_file_index_commit (GstIndex * _index, gint _writer_id)
       (GHFunc) _file_index_id_save_xml, writers);
 
   if (mkdir (index->location, 0777) && errno != EEXIST) {
-    GST_ERROR_OBJECT (index, "mkdir %s: %s", index->location, strerror (errno));
+    GST_ERROR_OBJECT (index, "mkdir %s: %s", index->location,
+        g_strerror (errno));
     return;
   }
 
@@ -800,13 +801,13 @@ gst_file_index_add_association (GstIndex * index, GstIndexEntry * entry)
   if (exact) {
     /* maybe overwrite instead? */
     GST_DEBUG_OBJECT (index,
-        "Ignoring duplicate index association at %lld",
+        "Ignoring duplicate index association at %" G_GINT64_FORMAT,
         GST_INDEX_ASSOC_VALUE (entry, 0));
     return;
   }
 
   {
-    gchar row_data[ARRAY_ROW_SIZE (id_index)];
+    gchar *row_data = (gchar *) g_malloc (ARRAY_ROW_SIZE (id_index));
     gint fx;
 
     gint32 flags_host = GST_INDEX_ASSOC_FLAGS (entry);
@@ -820,6 +821,8 @@ gst_file_index_add_association (GstIndex * index, GstIndexEntry * entry)
     }
 
     g_array_insert_vals (id_index->array, mx, row_data, 1);
+
+    g_free (row_data);
   }
 }
 
@@ -843,7 +846,7 @@ show_entry (GstIndexEntry *entry)
 
       g_print ("%d: %08x ", entry->id, GST_INDEX_ASSOC_FLAGS (entry));
       for (i = 0; i < GST_INDEX_NASSOCS (entry); i++) {
-        g_print ("%d %lld ", GST_INDEX_ASSOC_FORMAT (entry, i),
+        g_print ("%d %" G_GINT64_FORMAT, GST_INDEX_ASSOC_FORMAT (entry, i),
                              GST_INDEX_ASSOC_VALUE (entry, i));
       }
       g_print ("\n");
